/*
Invoicing and invoice/credit note document creation configuration
Create invoices on dispatch for Creditcards 
Create invoices on order for ISH_CASH_IN_ADVANCE, ISH_CREDITCARD, ISH_CASH_ON_DELIVERY, ISH_INVOICE, ISH_DEBIT_TRANSFER, ISH_ONLINEPAY, ISH_FASTPAY, ISH_INVOICE_TOTAL_ZERO
Create credit notes for all payment methods on return
*/
DO $$
DECLARE
  shopRef							int8 = 		(select id from oms."ShopDO" where "internalShopName"='inTRONICS_Business');
  supplierRef						int8;
  paymentProviderRef				int8 =		3;
  eventControlQueue					int8 = 		(select id from "ProcessesDO" where "processDefRef" = (select id from "ProcessDefDO" where "queueName" = 'EventControlQueue'));
  checkOrderQueue					int8 = 		(select id from "ProcessesDO" where "processDefRef" = (select id from "ProcessDefDO" where "queueName" = 'CheckOrderQueue'));
  announceOrderQueue				int8 = 		(select id from "ProcessesDO" where "processDefRef" = (select id from "ProcessDefDO" where "queueName" = 'AnnounceOrderQueue'));
  closeReturnQueue					int8 = 		(select id from "ProcessesDO" where "processDefRef" = (select id from "ProcessDefDO" where "queueName" = 'CloseReturnQueue'));
  returnTransmissionRequiredQueue	int8 = 		(select id from "ProcessesDO" where "processDefRef" = (select id from "ProcessDefDO" where "queueName" = 'ReturnTransmissionRequiredQueue'));
  closeDispatchQueue				int8 = 		(select id from "ProcessesDO" where "processDefRef" = (select id from "ProcessDefDO" where "queueName" = 'CloseDispatchQueue'));
  dispatchTransmissionRequiredQueue	int8 = 		(select id from "ProcessesDO" where "processDefRef" = (select id from "ProcessDefDO" where "queueName" = 'DispatchTransmissionRequiredQueue'));
  objectTypeDefRefOrder				int8 = 		(select id from "ObjectTypeDefDO" where "name" = 'Order');
  objectTypeDefRefReturn			int8 = 		(select id from "ObjectTypeDefDO" where "name" = 'Return');
  objectTypeDefRefDispatch			int8 = 		(select id from "ObjectTypeDefDO" where "name" = 'Dispatch');
  transmissionTypeDefRef			int8 =		320;
  processControlConfigRef			int8;
  processControlLineConfigRef		int8;
  eventRegistryEntryRefOrder		int8;
  eventRegistryEntryRefReturn		int8;
  eventRegistryEntryRefDispatch		int8;
  eventDefRef						int8;
  paymentEventDefRef				int8;
  communicationRef					int8;
  communicationPartnerRef			int8;
  decisionBeanDefRef				int8;
  sendingPartnerReferrerRef			int8;
  receivingPartnerReferrerRef		int8;
  paymentNotificationActionDefRef	int8;
  paymentDefRef						int8;
  shop2PaymentProvider2PaymentRef	int8;
  invoicingEventDefRef				int8 = 1;
  doProcess							boolean;
  doManualApprove					boolean;
  invoicingTypeDefRef			int8;
  

BEGIN
		
--#########################################################################
--# Reconfiguration of "CloseInvoicingPC"
--# Add new gate to trigger event based processing using EventManagerBean 
--# Add existing gates to trigger existing status based processes
--#########################################################################

-- add process control config for checkOrderQueue
processControlConfigRef = admin.add_process_control_config(objectTypeDefRefOrder, checkOrderQueue, shopRef, supplierRef);
-- add process control line config for event based processing --> target eventControlQueue
processControlLineConfigRef = admin.add_process_control_line_config(processControlConfigRef, objectTypeDefRefOrder, NULL, eventControlQueue, NULL, NULL, 3);
-- add process control line config for existing gate InvoicingTransmissionSyncQueue and status based processing
processControlLineConfigRef = admin.add_process_control_line_config(processControlConfigRef, objectTypeDefRefOrder, announceOrderQueue, announceOrderQueue, NULL, NULL, 1);

-- add process control config for closeReturnQueue
processControlConfigRef = admin.add_process_control_config(objectTypeDefRefReturn, closeReturnQueue, shopRef, supplierRef);
-- add process control line config for event based processing --> target eventControlQueue
processControlLineConfigRef = admin.add_process_control_line_config(processControlConfigRef, objectTypeDefRefReturn, NULL, eventControlQueue, NULL, NULL, 3);
-- add process control line config for existing gate InvoicingTransmissionSyncQueue and status based processing
processControlLineConfigRef = admin.add_process_control_line_config(processControlConfigRef, objectTypeDefRefReturn, returnTransmissionRequiredQueue, returnTransmissionRequiredQueue, NULL, NULL, 1);

-- add process control config for closeDispatchQueue
processControlConfigRef = admin.add_process_control_config(objectTypeDefRefDispatch, closeDispatchQueue, shopRef, supplierRef);
-- add process control line config for event based processing --> target eventControlQueue
processControlLineConfigRef = admin.add_process_control_line_config(processControlConfigRef, objectTypeDefRefDispatch, NULL, eventControlQueue, NULL, NULL, 3);
-- add process control line config for existing gate InvoicingTransmissionSyncQueue and status based processing
processControlLineConfigRef = admin.add_process_control_line_config(processControlConfigRef, objectTypeDefRefDispatch, dispatchTransmissionRequiredQueue, dispatchTransmissionRequiredQueue, NULL, NULL, 1);


--###########################################################
--# Event registry configuration
--# Trigger Invoicing Event Manager on 
--# CloseOrderQueue, CloseReturnQueue, CloseDispatchQueue
--###########################################################
eventDefRef := 3; -- invoicing event
eventRegistryEntryRefOrder = admin.add_event_registry_entry(checkOrderQueue, shopRef, eventDefRef, 'Invoicing Events on CloseOrderQueue');
eventRegistryEntryRefReturn = admin.add_event_registry_entry(closeReturnQueue, shopRef, eventDefRef, 'Invoicing Events on CloseReturnQueue');
eventRegistryEntryRefDispatch = admin.add_event_registry_entry(closeDispatchQueue, shopRef, eventDefRef, 'Invoicing Events on CloseDispatchQueue');


--#############################################################
--# Invoicing Event registry configuration
--#############################################################
paymentEventDefRef := 10; -- CREATE_PAYMENTNOTIFICATION
decisionBeanDefRef := 70; -- PAYMENT_ACTION_DECIDER_BEAN

-- Invoice for Creditcards (3) on dispatch
paymentDefRef := 3;
invoicingTypeDefRef := 1; -- invoice
decisionBeanDefRef := 120; -- INVOICING_DECIDER_BEAN
perform admin.add_invoicing_event_registry_entry (eventRegistryEntryRefDispatch, invoicingEventDefRef, paymentDefRef, invoicingTypeDefRef, decisionBeanDefRef);

-- Invoice for ISH_CASH_IN_ADVANCE, ISH_CASH_ON_DELIVERY, ISH_INVOICE, ISH_DEBIT_TRANSFER, ISH_ONLINEPAY, ISH_FASTPAY, ISH_INVOICE_TOTAL_ZERO on order
invoicingTypeDefRef := 1; -- invoice
decisionBeanDefRef := 120; -- INVOICING_DECIDER_BEAN
paymentDefRef := 2; -- ISH_CASH_IN_ADVANCE
perform admin.add_invoicing_event_registry_entry (eventRegistryEntryRefOrder, invoicingEventDefRef, paymentDefRef, invoicingTypeDefRef, decisionBeanDefRef);
paymentDefRef := 4; -- ISH_CASH_ON_DELIVERY
perform admin.add_invoicing_event_registry_entry (eventRegistryEntryRefOrder, invoicingEventDefRef, paymentDefRef, invoicingTypeDefRef, decisionBeanDefRef);
paymentDefRef := 5; -- ISH_INVOICE
perform admin.add_invoicing_event_registry_entry (eventRegistryEntryRefOrder, invoicingEventDefRef, paymentDefRef, invoicingTypeDefRef, decisionBeanDefRef);
paymentDefRef := 7; -- ISH_DEBIT_TRANSFER
perform admin.add_invoicing_event_registry_entry (eventRegistryEntryRefOrder, invoicingEventDefRef, paymentDefRef, invoicingTypeDefRef, decisionBeanDefRef);
paymentDefRef := 10; -- ISH_ONLINEPAY
perform admin.add_invoicing_event_registry_entry (eventRegistryEntryRefOrder, invoicingEventDefRef, paymentDefRef, invoicingTypeDefRef, decisionBeanDefRef);
paymentDefRef := 24; -- ISH_FASTPAY
perform admin.add_invoicing_event_registry_entry (eventRegistryEntryRefOrder, invoicingEventDefRef, paymentDefRef, invoicingTypeDefRef, decisionBeanDefRef);
paymentDefRef := 100; -- ISH_INVOICE_TOTAL_ZERO
perform admin.add_invoicing_event_registry_entry (eventRegistryEntryRefOrder, invoicingEventDefRef, paymentDefRef, invoicingTypeDefRef, decisionBeanDefRef);

-- Credit note for all payment methods on return
paymentDefRef := NULL;
invoicingTypeDefRef := 2; -- credit
decisionBeanDefRef := 120; -- INVOICING_DECIDER_BEAN
perform admin.add_invoicing_event_registry_entry (eventRegistryEntryRefDispatch, invoicingEventDefRef, paymentDefRef, invoicingTypeDefRef, decisionBeanDefRef);

--############################
--# Invoice number generation
--############################ 
-- Nummerngenerierung mit LONG_NO_SEQUENCER fuer Rechnungen und Gutschriften innerhalb des gleichen Nummernkreises
invoicingTypeDefRef := 2; -- credit
INSERT INTO "InvoicingNoConfigDO"(id, "name", "countGenerated", "creationDate", enabled, "generationDate", "startNumber", "endNumber", "numberRangeFormatterDefRef", "invoicingTypeDefRef", "lastGeneratedNumber", "maxNotAllocatedInvoiceNo", "modificationDate", "startDate","shopRef", version)
    SELECT nextval('"InvoicingNoConfigDO_id_seq"'), 'Invoicing number generation', 0, CURRENT_TIMESTAMP, true, null, '2000000000', '2999999999', 5, invoicingTypeDefRef, null, 5, CURRENT_TIMESTAMP, '2017-02-24 00:00:00', shopRef, 1
	WHERE NOT EXISTS (select * from "InvoicingNoConfigDO" where (invoicingTypeDefRef,shopRef)=(invoicingTypeDefRef, shopRef));
invoicingTypeDefRef := 1; -- invoice
INSERT INTO "InvoicingNoConfigDO"(id, "name", "countGenerated", "creationDate", enabled, "generationDate", "startNumber", "endNumber", "numberRangeFormatterDefRef", "invoicingTypeDefRef", "lastGeneratedNumber", "maxNotAllocatedInvoiceNo", "modificationDate", "startDate","shopRef", version)
    SELECT nextval('"InvoicingNoConfigDO_id_seq"'), 'Invoicing number generation', 0, CURRENT_TIMESTAMP, true, null, '2000000000', '2999999999', 5, invoicingTypeDefRef, null, 5, CURRENT_TIMESTAMP, '2017-02-24 00:00:00', shopRef, 1
	WHERE NOT EXISTS (select * from "InvoicingNoConfigDO" where ("invoicingTypeDefRef","shopRef")=(invoicingTypeDefRef, shopRef));

--#########################################################################
--# Communication partner configuration using DOCUMENT_CREATION_SYNCH_BEAN
--# Sending partner --> Shop
--# Receiving partner --> Paymentprovider
--#########################################################################
sendingPartnerReferrerRef := (select id from "PartnerReferrerDO" where "shopRef" = shopRef);
receivingPartnerReferrerRef := (select id from "PartnerReferrerDO" where "shopRef" = shopRef);
communicationRef := (select "id" from "CommunicationDO" where "executionBeanDefRef"=65 and "transmissionTypeDefRef"=300 and "documentFormatDefRef"=1);
communicationPartnerRef = admin.add_communication_partner(NULL, false, communicationRef, sendingPartnerReferrerRef, receivingPartnerReferrerRef);

--#########################################################
--# Execution bean values for DOCUMENT_CREATION_SYNCH_BEAN
--#########################################################
perform admin.add_execution_bean_value(213, 'PDF', communicationPartnerRef);
perform admin.add_execution_bean_value(214, 'INVOICE_CREDIT_NOTE', communicationPartnerRef);
perform admin.add_execution_bean_value(215, 'true', communicationPartnerRef);
perform admin.add_execution_bean_value(213, 'PDF', communicationPartnerRef);

--#############################
--# Document transfomer config
--#############################
perform admin.add_document_transformer_config(1, 2, 17, shopRef, 2, true);
perform admin.add_document_transformer_config(1, 2, 18, shopRef, 2, true);
perform admin.add_document_transformer_config(1, 2, 19, shopRef, 2, true);

END;
$$;